@@ -35,6 +35,7 @@ import cPickle
 import zlib
 from factors import Factor, _find_levels
 import sys
+import itertools
 
 class DataFrame(object):
     """An implemention of an almost R like DataFrame object.
@@ -188,9 +189,6 @@ class DataFrame(object):
         sorted_df = other[order,(new_columns)]
         return self.cbind_view(sorted_df)
 
-
-
-
     def insert_column(self, column_name, values, position='last'):
         """Insert a new column into your DataFrame.
         
@@ -234,6 +232,8 @@ class DataFrame(object):
         """
         if old_name == new_name:
             return
+        if new_name in self.value_dict: 
+            raise ValueError("Trying to rename %s into already existing column name %s. Drop %s first" % (old_name, new_name, new_name))
         old_name = self._map_column_number_to_name(old_name)
         try:
             self.value_dict[new_name] = self.value_dict[old_name]
@@ -422,8 +422,10 @@ class DataFrame(object):
             except KeyError:
                 pass
         try:
-            row_name = int(row_name)
-            return row_name
+            row_idx = int(row_name)
+            if row_idx > len(self):
+                raise KeyError("")
+            return row_idx
         except:
             if self._row_names_dict:
                 raise KeyError("Could not find rowname: %s" % row_name)
@@ -737,6 +739,24 @@ class DataFrame(object):
         for column_name in self.columns_ordered:
             yield self.value_dict[column_name]
 
+    def groupby(self, column_name):
+        """Yield (value, sub_df) for all values of column column_name, where sub_df[:,'column_name'] == value for each subset.
+        Basically, itertools.groupby for dataframes.
+        No prior sorting necessary.
+        
+        """
+        sorted_df = self.sort_by(column_name)
+        column = sorted_df.get_column_view(column_name)
+        pos, = numpy.where(column[1:] != column[:-1])
+        pos = numpy.concatenate(([0],pos+1,[len(column)]))
+        for start, stop in itertools.izip(pos[:-1],pos[1:]):
+            value = column[start]
+            sub_df = sorted_df[start:stop, :]
+            yield (value, sub_df)
+
+
+
+
     def __len__(self):
         """Return the row count of the DataFrame."""
         return self.num_rows
@@ -806,7 +826,8 @@ class DataFrame(object):
             other = str(other)
 
         try:
-            return self.value_dict[self.columns_ordered[0]] == other
+            #return self.value_dict[self.columns_ordered[0]] == other
+            return numpy.equal(self.value_dict[self.columns_ordered[0]], other)
         except AttributeError, e:
             if str(e) == "'bool' object has no attribute 'view'" and other is None:
                 #we have run into a bug in the masked array... let's do this by hand
@@ -1088,6 +1109,11 @@ class DataFrame(object):
             vd[key] = self.value_dict[key].copy()
         return DataFrame(vd, self.columns_ordered[:], self.row_names)
 
+    def shallow_copy(self):
+        """Return a shallow copy of the DataFrame - ie. shared data columns, but differing objects"""
+        return DataFrame(self.value_dict, self.columns_ordered[:], self.row_names)
+
+
     def types(self):
         """Return a tuple of the types of the DataFrame"""
         return tuple(((key, self.value_dict[key].dtype) for key in self.value_dict))
@@ -1219,6 +1245,37 @@ class DataFrame(object):
                 result.value_dict[id_var] = Factor(result.value_dict[id_var], self.value_dict[id_var].levels, False)
         return result
 
+    def aggregate(self, key_vars, aggregation_function):
+        """Iterate for every value combination of the key vars, call the aggregation_function with the sub-df. Take the returned dicts, turn them into a new df"""
+        if not hasattr(key_vars, '__iter__'):
+            raise ValueError("key_vars must be a list/iterable")
+        key_vars = tuple(key_vars)
+        df = self.sort_by(key_vars, [True] *len(key_vars))
+        last_key = None
+        last_start = 0
+        res = []
+        for ii in xrange(0, len(df)):
+            key = tuple(df[ii, key_vars].get_row_as_list(0))
+            if key != last_key:
+                if last_key != None:
+                    sub_df = df[last_start:ii,:]
+                    r = aggregation_function(
+                        sub_df)
+                    for k in key_vars:
+                        r[k] = df.get_value(ii -1, k)
+                    res.append(r)
+                    last_start = ii 
+                last_key = key
+        if last_start != len(df):
+            r = aggregation_function(
+                        df[last_start:,:])
+            for k in key_vars:
+                r[k] = df.get_value(len(df) -1, k)
+            res.append(r)
+        return DataFrame(res)
+
+
+
     def sort_by_minimal_distance(self, columns_to_consider, distance_function = 'euclid'):
         if distance_function == 'euclid':
             def euclid(a, b):
@@ -1272,7 +1329,10 @@ class _RowNamesAttribute(object):
             if isinstance(value, DataFrame):
                 value._assert_single_columned()
                 value = value.get_column_view(0)
-            obj._row_names_ordered = numpy.array(value)
+            arr = numpy.array(value)
+            if arr.dtype.kind == 'i' or arr.dtype.kind == 'u':
+                raise ValueError("Integer rownames are not supported")
+            obj._row_names_ordered = arr
             self.update_dictionary(obj)
         else:
             obj._row_names_ordered = None
@@ -1310,13 +1370,31 @@ def DataFrameFrom2dArray(array, column_names, row_names = None):
 
 def combine(dataframe_generator):
     """rowbind multiple dataframes into one"""
-    df = None
-    for a_df in dataframe_generator:
-        if df is None:
-            df = a_df
+    all_dfs = list(dataframe_generator)
+    total_row_count = 0
+    columns = set()
+    dtypes = {}
+    for df in all_dfs:
+        if not columns:
+            columns = set(df.columns_ordered)
         else:
-            df = df.rbind_copy(a_df)
-    return df
+            if columns != set(df.columns_ordered):
+                raise KeyError("Dataframes had unequal columns")
+        for column_name in df.columns_ordered:
+            if not column_name in dtypes:
+                dtypes[column_name] = df.get_column_view(column_name).dtype
+            else:
+                dtypes[column_name] = numpy.find_common_type([], [dtypes[column_name], df.get_column_view(column_name).dtype])
+        total_row_count += len(df)
+    value_dict = {}
+    for column_name in columns:
+        value_dict[column_name] = numpy.ma.zeros((total_row_count, ), dtype=dtypes[column_name])
+    offset = 0
+    for df in all_dfs:
+        for column_name in columns:
+            value_dict[column_name][offset: offset + len(df)] = df.value_dict[column_name]
+        offset += len(df)
+    return DataFrame(value_dict, all_dfs[0].columns_ordered)
 
 
 def _convert_to_numpy(seq, column_name):
@@ -1483,7 +1561,14 @@ try:
                 all_str = True
                 all_bool = True
                 for value in o:
-                    if not type(value) is str and not type(value) is unicode and not type(value) is numpy.string_ and not (type(value) is numpy.ma.core.MaskedArray and value.mask == True):
+                    if (
+                            not type(value) is str and 
+                            not type(value) is unicode and 
+                            not type(value) is numpy.string_ and 
+                            not (type(value) is numpy.ma.core.MaskedArray and value.mask == True) and
+                            not (type(value) is numpy.ma.core.MaskedConstant and value.mask == True)
+                                
+                                ):
                         all_str = False
                         break
                     if not type(value) is bool or type(value) is numpy.bool_:
@@ -1528,7 +1613,7 @@ try:
                     names.append(str(column_name))
                     parameters.append(numpy2ri_vector(col))
                 except ValueError, e:
-                    raise ValueError(str(e) + ' Offending column: %s, dtype' % column_name, col.dtype)
+                    raise ValueError(str(e) + ' Offending column: %s, dtype: %s, content: %s' %( column_name, col.dtype, col[:10]))
             if not o.row_names is None:
                 kw_params['row.names'] = numpy2ri_vector(o.row_names)
             #print parameters
