@@ -9,6 +9,8 @@ import math
 import os
 import numpy
 import sqlite3
+import codecs
+import tempfile
 
 from dataframe import DataFrame
 from factors import Factor
@@ -59,7 +61,7 @@ def _unquote_single_value(quote_char, value):
         return value
 
 
-def _open_file(filename_or_file_object, mode = 'rb'):
+def _open_file(filename_or_file_object, mode = 'rb', encoding=None):
     """Smartly open a file.
 
         Takes either an open file (returning it),
@@ -72,11 +74,80 @@ def _open_file(filename_or_file_object, mode = 'rb'):
         if filename_or_file_object.endswith('.gz'):
             filehandle = gzip.GzipFile(filename_or_file_object, mode)
         else:
-            filehandle = open(filename_or_file_object, mode)
+            if encoding:
+                filehandle = codecs.open(filename_or_file_object, mode, encoding=encoding)
+            else:
+                filehandle = open(filename_or_file_object, mode)
     else:
         filehandle = filename_or_file_object
     return filehandle
 
+def read(filename): 
+    """Try to read a df from a file, guessing by file extension which parser to use"""
+    if filename.endswith('.xls'):
+        import xlrd
+        try:
+            df = DF2Excel.read(filename)
+        except xlrd.XLRDError:
+            df = DF2TSV().read(filename)
+    elif filename.endswith('.tsv'): 
+        df = DF2TSV().read(filename)
+    elif filename.endswith('.csv'): 
+        df = DF2CSV().read(filename)
+    elif filename.endswith('.xlsx'):
+        df = DF2XLSX().read(filename)
+    else:
+        raise ValueError("Don't know how to parse %s" % filename)
+    return df
+
+def write(df, outputfilename):
+    """Decide upon file extension how to write this dataframe..."""
+    if outputfilename.endswith('.tsv'):
+        DF2CSV().write(df, outputfilename, dialect=TabDialect())
+    elif outputfilename.endswith('.csv'):
+        DF2CSV().write(df, outputfilename)
+    elif outputfilename.endswith('.xls'):
+        DF2Excel().write(df, outputfilename)
+    elif filename.endswith('.xlsx'):
+        DF2XLSX().write(df, outputfilename)
+    else:
+        raise ValueError("Don't know how to handle %s " % outputfilename)
+
+def parse_md_string(md_string):
+    """Parse an MD string like and return a list of positions where it mismatched the reference
+    deletions raise an exception for now...
+    '36' (36 straight matches
+    '10A25' (10 matches, an A that is mismatched, 25 letters matched
+    '24A4A6' (24 matches, an A, 4 more letters, a mismatched A, 6 more letters
+    '10A5^AC6' (10 matches, an A, 5 matches, a deletion of AC, 6 more letters
+    (how are insertiens represented?)
+    
+    Unfortunatly, there is no way to capture this with a single regexps (it can be validated
+    with the one in the SAM format spec, but python only keeps the last group of nested groups...)
+    So a true lexer it is...
+    """
+    import re
+    tokens = re.findall("([0-9]+|[A-Z]+|^)", md_string)
+    tokens.reverse()
+    try:
+        pos = int(tokens.pop())
+    except:
+        raise ValueError("%s was not a valid md string, did not start with a number." % md_string)
+    res = []
+    while tokens:
+        h = tokens.pop()
+        if h == '^':
+            raise ValueError("parse_md_string currently does not handle deletions (and insertions)")
+        elif ord('0') <= ord(h[0]) <= ord('9'):
+            pos += int(h)
+        else:
+            for letter in h:
+                res.append(pos)
+                pos += 1
+    return res
+
+
+
 import cStringIO
 import codecs
 class UnicodeWriter:
@@ -115,7 +186,7 @@ class DF2CSV:
         skip_rows = 0, columns_to_include = None, columns_to_exclude = None, 
         na_text = "NA", type_hints = None, handle_quotes = False, 
         has_row_names = False, r_skipped_first_column_name = False, 
-        comment_character = None):
+        comment_character = None, encoding='latin-1', rename_duplicate_columns = False):
         """Read a csv file and turn it into a DataFrame.
         
         ::
@@ -131,30 +202,31 @@ class DF2CSV:
                     if it isn't it can be read faster.
             r_skipped_first_column_name = if you export from R with write.table(row.names=TRUE),
                 r skips the first column name. This handles this.
+            rename_duplicate_columns - if a column name occurs multiple times, the second one is labeled _2, and so on.
 
         """
         if isinstance(header, csv.Dialect):
             raise ValueError("You passed a cvs.Dialect as second parameter, but that needs to be a bool. Pass your dialect as third (or dialect=) parameter.")
         try:
-            filehandle = _open_file(filename_or_file_object,'rU') 
+            filehandle = _open_file(filename_or_file_object,'r', encoding=encoding) 
             return self.__encapsulated_read(filehandle,
                                            header, dialect, skip_rows, columns_to_include, 
                                            columns_to_exclude, na_text, type_hints, handle_quotes, 
-                                           has_row_names, r_skipped_first_column_name, comment_character)
+                                           has_row_names, r_skipped_first_column_name, comment_character, rename_duplicate_columns)
         except (zlib.error, IOError), e:
             if isinstance(e, zlib.error) or str(e).find('CRC check failed') != -1:
                 filehandle = _open_file(filename_or_file_object,'r') 
                 return self.__encapsulated_read(filehandle,
                                                header, dialect, skip_rows, columns_to_include, 
                                                columns_to_exclude, na_text, type_hints, handle_quotes, 
-                                               has_row_names, r_skipped_first_column_name, comment_character)
+                                               has_row_names, r_skipped_first_column_name, comment_character, rename_duplicate_columns)
             else:
                 raise e
 
     def __encapsulated_read(self, filehandle, header=True, dialect=None, skip_rows = 0, 
         columns_to_include = None, columns_to_exclude = None, na_text = "NA", type_hints = None, 
         handle_quotes = False, has_row_names = False, r_skipped_first_column_name = False, 
-        comment_character = None):
+        comment_character = None, rename_duplicate_columns = False):
         if dialect is None:
             dialect = ExcelDialect()
         if isinstance(dialect, str):
@@ -201,7 +273,18 @@ class DF2CSV:
                     should += 1
                 raise ValueError("Wrong number of headers. Should be %i, was %i (potentially including skipped first column)" % (should, len(fields)))
             if len(fields) != len(set(fields)):
-                raise ValueError("Some fields occur twice")
+                if rename_duplicate_columns:
+                    new_fields = []
+                    for f in fields:
+                        counter = 2
+                        org_f = f
+                        while f in new_fields:
+                            f = "%s_%i" % ( org_f, counter )
+                            counter += 1
+                        new_fields.append(f)
+                    fields = new_fields
+                else:
+                    raise ValueError("Some fields occur twice")
         else:
             num_fields = len(first_line)
             num_chars = int(math.ceil(math.log10(num_fields)))
@@ -251,7 +334,10 @@ class DF2CSV:
                     else:
                         results[field] = list(columns[i])
             except IndexError:
-                raise ValueError("Could not find column for field %s, file: %s" % (field, filehandle.name) )
+                if columns: #so you can import 'empty' dfs
+                    raise ValueError("Could not find column for field %s, file: %s" % (field, filehandle.name) )
+                else: 
+                    results[field] = []
         row_names = None
         if has_row_names:
             if isinstance(has_row_names, str): 
@@ -389,51 +475,70 @@ class DF2ARFF:
 class DF2Excel:
     """Use xlrd and xlwt to write 'real' excel files"""
 
-    def read(self, filename, sheet_name = None, row_offset = 0, col_offset = 0, row_max = None, col_max = None, filter_empty = False, NA_string = None, handle_encoding_errors = lambda u: u.encode('latin-1', 'replace').strip().decode('latin-1')):
+    def read(self, filename, sheet_name = None, row_offset = 0, col_offset = 0, row_max = None, col_max = None, filter_empty = False, NA_string = None, handle_encoding_errors = lambda u: u.encode('latin-1', 'replace').strip().decode('latin-1'), columns_to_include = None):
         import xlrd
-        wb = xlrd.open_workbook(filename)
-        if sheet_name is None:
-            ws = wb.sheet_by_index(0)
+        if filename.endswith('.gz'):
+            tf = tempfile.NamedTemporaryFile()
+            op = _open_file(filename,'rb')
+            tf.write(op.read())
+            op.close()
+            tf.flush()
+            filename = tf.name
         else:
-            if type(sheet_name) == int:
-                ws = wb.sheet_by_index(sheet_name)
+            tf = None
+        try:
+            wb = xlrd.open_workbook(filename)
+            if sheet_name is None:
+                ws = wb.sheet_by_index(0)
             else:
-                ws = wb.sheet_by_name(sheet_name)
-        if row_max is None:
-            row_max = ws.nrows
-        if col_max is None:
-            col_max = ws.ncols
-        cols = {}
-        col_names_in_order = []
-        for col_name in ws.row(0 + row_offset)[col_offset: col_max]:
-            name = col_name.value
-            counter = 2
-            while name in col_names_in_order:
-                name = "%s_%i" % ( col_name.value, counter )
-                counter += 1
-            col_names_in_order.append(name)
-            cols[name] = []
-        for row_no in xrange(0 + row_offset + 1, row_max):
-            row = ws.row(row_no)
-            found = False
-            for y in xrange(0 + col_offset, col_max):
-                value = row[y].value
-                try:
-                    if str(value).strip() == '' or row[y].ctype == xlrd.XL_CELL_EMPTY or row[y].ctype == xlrd.XL_CELL_ERROR:
-                        value = None
-      #              cols[col_names_in_order[y - col_offset]].append(str(value))
-                    if type(value) == unicode and value == NA_string:
-                        value = None
-                except UnicodeEncodeError:
-                    value = handle_encoding_errors(value)
-                cols[col_names_in_order[y - col_offset]].append((value))
-                if value:
-                    found = True
-            if not found and filter_empty:
-                for k in col_names_in_order:
-                    cols[k].pop()
-        return DataFrame(cols, col_names_in_order)
-
+                if type(sheet_name) == int:
+                    ws = wb.sheet_by_index(sheet_name)
+                else:
+                    ws = wb.sheet_by_name(sheet_name)
+            if row_max is None:
+                row_max = ws.nrows
+            if col_max is None:
+                col_max = ws.ncols
+            cols = {}
+            col_names_in_order = []
+            columns_used = []
+            for yy, col_name in enumerate(ws.row(0 + row_offset)[col_offset: col_max]):
+                name = col_name.value
+                counter = 2
+                while name in col_names_in_order:
+                    name = "%s_%i" % ( col_name.value, counter )
+                    counter += 1
+                col_names_in_order.append(name)
+                if not columns_to_include or name in columns_to_include:
+                    cols[name] = []
+                    columns_used.append(yy)
+            if not columns_used:
+                raise ValueError("No columns to import - columns_to_include did not reference any present. Column_to_include was %s, available: %s" % (columns_to_include, col_names_in_order))
+            if columns_to_include and set(columns_to_include).difference(col_names_in_order):
+                raise ValueError("Nonexistant column name(s) in columns_to_include: %s" % (set(columns_to_include).difference(col_names_in_order),))
+            for row_no in xrange(0 + row_offset + 1, row_max):
+                row = ws.row(row_no)
+                found = False
+                for y in  columns_used:#xrange(0 + col_offset, col_max):
+                    value = row[y].value
+                    try:
+                        if str(value).strip() == '' or row[y].ctype == xlrd.XL_CELL_EMPTY or row[y].ctype == xlrd.XL_CELL_ERROR:
+                            value = None
+          #              cols[col_names_in_order[y - col_offset]].append(str(value))
+                        if type(value) == unicode and value == NA_string:
+                            value = None
+                    except UnicodeEncodeError:
+                        value = handle_encoding_errors(value)
+                    cols[col_names_in_order[y - col_offset]].append((value))
+                    if value:
+                        found = True
+                if not found and filter_empty:
+                    for k in col_names_in_order:
+                        cols[k].pop()
+            return DataFrame(cols, [x for (yy,x) in enumerate(col_names_in_order) if yy in columns_used])
+        finally:
+            if tf:
+                tf.close()
 
 
     def write(self, data_frame_or_dict_of_dataframes, filename_or_file_object, sheet_name = "DataFrame",
@@ -516,6 +621,78 @@ class DF2Excel:
             except:
                 return unicode(value)[:65000]
 
+class DF2XLSX:
+    """Use openpyxl to read and xlxs excel files"""
+
+    def read(self, filename, sheet_name = None, row_offset = 0, col_offset = 0, row_max = None, col_max = None, filter_empty = False, NA_string = None, handle_encoding_errors = lambda u: u.encode('latin-1', 'replace').strip().decode('latin-1')):
+        import openpyxl
+        wb = openpyxl.reader.excel.load_workbook(filename)
+        if sheet_name is None:
+            ws = wb.worksheets[0]
+        else:
+            if type(sheet_name) == int:
+                ws = wb.worksheets[sheet_name]
+            else:
+                ws = wb.sheet_by_name(sheet_name)
+        if row_max is None:
+            row_max = len(ws.rows)
+        if col_max is None:
+            col_max = len(ws.columns)
+        cols = {}
+        col_names_in_order = []
+        for col_name in ws.rows[0 + row_offset][col_offset: col_max]:
+            name = col_name.value
+            counter = 2
+            while name in col_names_in_order:
+                name = "%s_%i" % ( col_name.value, counter )
+                counter += 1
+            col_names_in_order.append(name)
+            cols[name] = []
+        for row_no in xrange(0 + row_offset + 1, row_max):
+            row = ws.rows[row_no]
+            found = False
+            for y in xrange(0 + col_offset, col_max):
+                value = row[y].value
+                try:
+                    if str(value).strip() == '': #or row[y].ctype == xlrd.XL_CELL_EMPTY or row[y].ctype == xlrd.XL_CELL_ERROR:
+                        value = None
+      #              cols[col_names_in_order[y - col_offset]].append(str(value))
+                    if type(value) == unicode and value == NA_string:
+                        value = None
+                except UnicodeEncodeError:
+                    value = handle_encoding_errors(value)
+                cols[col_names_in_order[y - col_offset]].append((value))
+                if value:
+                    found = True
+            if not found and filter_empty:
+                for k in col_names_in_order:
+                    cols[k].pop()
+        return DataFrame(cols, col_names_in_order)
+
+    def write(self, data_frame_or_dict_of_dataframes , filename_or_file_object, sheet_name = "DataFrame",
+             highlight_columns = None):
+        import openpyxl
+        wb = openpyxl.workbook.Workbook()
+        if not isinstance(data_frame_or_dict_of_dataframes, dict):
+            data_frame_or_dict_of_dataframes = {sheet_name: data_frame_or_dict_of_dataframes}
+        first = True
+        for sheet_name, data_frame in data_frame_or_dict_of_dataframes.items():
+            if first:
+                ws = wb.get_active_sheet()
+                first = False
+            else:
+                ws = wb.create_sheet()
+            ws.title = sheet_name
+            for xx, column_name in enumerate(data_frame.columns_ordered):
+                ws.cell(row = 0, column = xx).value = column_name
+                for yy, value in enumerate(data_frame.get_column_view(column_name)):
+                    try:
+                        ws.cell(row = yy + 1, column = xx).value = value
+                    except TypeError:
+                        ws.cell(row = yy + 1, column = xx).value = float(value)
+        wb.save(filename_or_file_object)
+
+
 class DF2Sqlite:
 
     def write(self, data_frame_or_dict_of_dataframes, filename_or_file_object, sheet_name = 'DataFrame'):
