@@ -31,11 +31,12 @@
 import unittest
 import os
 import numpy
-from dataframe import DataFrame, DataFrameFrom2dArray
+from dataframe import DataFrame, DataFrameFrom2dArray, combine
 from parsers import TabDialect, DF2CSV, DF2ARFF, DF2Excel, DF2TSV
 from factors import Factor
 import cPickle
 import StringIO
+import tempfile
 
 
 class DataFrameTests(unittest.TestCase):
@@ -63,6 +64,18 @@ class DataFrameTests(unittest.TestCase):
         self.assertEqual(c.get_row(4), {'Field1': 5, 'Field2': 'def'})
         self.assertEqual(c.get_row(5), {'Field1': 6, 'Field2': 'hgi'})
 
+    def test_combine(self):
+        a = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
+        b = DataFrame( { "Field1": [4, 5,6], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
+        c = combine([a, b])
+        self.assertEqual(c.num_rows, 6)
+        self.assertEqual(c.get_row(0), {'Field1': 1, 'Field2': 'abc'})
+        self.assertEqual(c.get_row(1), {'Field1': 2, 'Field2': 'def'})
+        self.assertEqual(c.get_row(2), {'Field1': 3, 'Field2': 'hgi'})
+        self.assertEqual(c.get_row(3), {'Field1': 4, 'Field2': 'abc'})
+        self.assertEqual(c.get_row(4), {'Field1': 5, 'Field2': 'def'})
+        self.assertEqual(c.get_row(5), {'Field1': 6, 'Field2': 'hgi'})
+
     def testRBindMultiple(self):
         a = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
         b = DataFrame( { "Field1": [4, 5,6], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
@@ -81,8 +94,28 @@ class DataFrameTests(unittest.TestCase):
     def testRBindRequiresSameFields(self):
         def inner():
             a = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
-            b = DataFrame( { "Field1": [4, 5,6]}, ['Field1','Field2'])
+            b = DataFrame( { "Field1": [4, 5,6]}, ['Field1'])
             c = a.rbind_copy(b, b)
+        self.assertRaises(KeyError, inner)
+
+    def testCombineRequiresSameFields(self):
+        def inner():
+            a = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
+            b = DataFrame( { "Field1": [4, 5,6]}, ['Field1'])
+            c = combine([a, b])
+        self.assertRaises(KeyError, inner)
+
+    def test_combine_promots_int_to_float(self):
+        a = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
+        b = DataFrame( { "Field1": [4.3, 5.5,6.6], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
+        c = combine([a, b])
+        self.assertEqual(c.num_rows, 6)
+        self.assertEqual(c.get_row(0), {'Field1': 1.0, 'Field2': 'abc'})
+        self.assertEqual(c.get_row(1), {'Field1': 2.0, 'Field2': 'def'})
+        self.assertEqual(c.get_row(2), {'Field1': 3.0, 'Field2': 'hgi'})
+        self.assertEqual(c.get_row(3), {'Field1': 4.3, 'Field2': 'abc'})
+        self.assertEqual(c.get_row(4), {'Field1': 5.5, 'Field2': 'def'})
+        self.assertEqual(c.get_row(5), {'Field1': 6.6, 'Field2': 'hgi'})
 
     def testRBindSuceedsWithMaskedInts(self):
         a = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
@@ -91,6 +124,13 @@ class DataFrameTests(unittest.TestCase):
         for x, y in zip(c.get_column_view('Field1').mask, [False, False, False, False, True, False]):
             self.assertEqual(x,y)
 
+    def testCombineSuceedsWithMaskedInts(self):
+        a = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
+        b = DataFrame( { "Field1": [1, None,3], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
+        c = combine([a, b])
+        for x, y in zip(c.get_column_view('Field1').mask, [False, False, False, False, True, False]):
+            self.assertEqual(x,y)
+
     def testRBindSuceedsWithMaskedStrings(self):
         a = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
         b = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc',None,'hgi']}, ['Field1','Field2'])
@@ -189,6 +229,15 @@ class DataFrameTests(unittest.TestCase):
                 x = a.join_columns_on(b, 'Field2', 'Field2')
             self.assertRaises(ValueError, inner) 
 
+    def test_groupby(self):
+        a = DataFrame( { "Field1": [1, 2,1], "Field2": ['shu','def','hgi']}, ['Field1','Field2'])
+        b = list(a.groupby('Field1'))
+        self.assertEqual(b[0][0], 1)
+        self.assertTrue((b[0][1].get_column_view('Field2') == ['shu','hgi']).all())
+        self.assertEqual(b[1][0], 2)
+        self.assertTrue((b[1][1].get_column_view('Field2') == ['def']).all())
+
+
     def testInsertColumn(self):
         a = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc','def','hgi']}, ['Field1','Field2'])
         a.insert_column('test', [7.0,8.0,9.0])
@@ -328,6 +377,19 @@ class DataFrameTests(unittest.TestCase):
             {"Field2": 'hgi', 'source': 'Field3', 'value': 30}])
         self.assertEqual(x,supposed_melted)
 
+    def test_simple_aggregate(self):
+        a = DataFrame( { "Field1": [1, 2,3], "Field2": ['shu','def','hgi'], 'Field3': [10,20,30]}, ['Field1','Field2'])
+        x = a.melt(["Field2"], ["Field1",'Field3'], 'source','value')
+        y = x.aggregate(['Field2'], lambda d: {'sum': sum(d.get_column_view('value'))})
+        supposed_aggregated = DataFrame([
+            {'Field2': 'def', 'sum': 22},
+            {'Field2': 'hgi', 'sum': 33},
+            {'Field2': 'shu', 'sum': 11},
+            ])
+        self.assertEqual(y, supposed_aggregated)
+
+
+
     def test_level_melt(self):
         a = DataFrame( { "Field1": [1, 2,3], "Field2": ['shu','def','hgi'], 'Field3': [10,20,30]}, ['Field1','Field2'])
         a.turn_into_level('Field2')
@@ -1129,6 +1191,12 @@ class DataFrameManipulationTests(unittest.TestCase):
         self.assertFalse('Field1' in a.value_dict)
         self.assertTrue('Field3' in a.value_dict)
 
+    def testRenamingByNameRaisesOnDuplicate(self):
+        a = DataFrame( { "Field1": [1, 2,3,4,5,6], 'Field3': [7.0,8.0,9.0,10.0,11.0,12.0]},['Field1','Field3'])
+        def inner():
+            a.rename_column('Field1', 'Field3')
+        self.assertRaises(ValueError, inner)
+
     def testRenamingByPosition(self):
         a = DataFrame( { "Field1": [1, 2,3,4,5,6], 'Field3': [7.0,8.0,9.0,10.0,11.0,12.0]},['Field1','Field3'])
         a.rename_column(0, 'FieldX')
@@ -1331,7 +1399,13 @@ class DataFrameRowNamesTests(unittest.TestCase):
             ["One", "Two","Three"]
             , ["A","B"]))
 
-
+    def testSettingIntegerRownamesRaises(self):
+        a = DataFrame( { "Field1": [1, 2,3], "Field2": ['abc','def','hgi'], 'Field3': [7.0,8.0,9.0]}, ['Field1','Field2', 'Field3'])
+        a.row_names = [ 'a','b','c']
+        def inner():
+            a.row_names = [55,66,77]
+        self.assertEqual(a.get_value('a','Field1'), 1)
+        self.assertRaises(ValueError, inner)
 
 class LevelTests(unittest.TestCase):
   
@@ -1500,7 +1574,85 @@ class DataFramePicklingTests(unittest.TestCase):
         op.seek(0,0)
         b = cPickle.load(op)
         self.assertEqual(a, b)
-       
+
+class DF2ExcelTests(unittest.TestCase):
+    def test_read_write(self):
+        a = DataFrame ( {
+            "one": [1, 2,3,4], #mask = False
+            'two': [1.0,2.0,3.0,4.0], #mask = True
+            'three': [True, True, False, False], #no mask set... = False
+             'four': ['sha','shu','shum','shim'], #some mask...
+            'five': numpy.array([1,2,3,4], dtype=numpy.float128)})
+        nf = tempfile.NamedTemporaryFile()
+        DF2Excel().write(a, nf)
+        nf.seek(0,0)
+        nf.flush()
+        b = DF2Excel().read(nf.name)
+        nf.close()
+        self.assertEqual(a, b)
+
+    def test_read_write_mixed_type(self):
+        a = DataFrame ( {
+            "one": [1, 2,3,4], #mask = False
+            'two': [1.0,2.0,3.0,'SHu'], #mask = True
+            'three': [True, True, False, False], #no mask set... = False
+             'four': ['sha','shu','shum','shim'], #some mask...
+            'five': numpy.array([1,2,3,4], dtype=numpy.float128)})
+        nf = tempfile.NamedTemporaryFile()
+        DF2Excel().write(a, nf)
+        nf.seek(0,0)
+        nf.flush()
+        b = DF2Excel().read(nf.name)
+        nf.close()
+        self.assertEqual(a, b)
+
+
+    def test_reading_sub_columns(self):
+        a = DataFrame ( {
+            "one": [1, 2,3,4], #mask = False
+            'two': [1.0,2.0,3.0,'SHu'], #mask = True
+            'three': [True, True, False, False], #no mask set... = False
+             'four': ['sha','shu','shum','shim'], #some mask...
+            'five': numpy.array([1,2,3,4], dtype=numpy.float128)})
+        nf = tempfile.NamedTemporaryFile()
+        DF2Excel().write(a, nf)
+        nf.seek(0,0)
+        nf.flush()
+        b = DF2Excel().read(nf.name, columns_to_include=['three','four'])
+        nf.close()
+        self.assertEqual(a[:,('three','four')], b)
+
+    def test_raises_on_removing_all(self):
+        a = DataFrame ( {
+            "one": [1, 2,3,4], #mask = False
+            'two': [1.0,2.0,3.0,'SHu'], #mask = True
+            'three': [True, True, False, False], #no mask set... = False
+             'four': ['sha','shu','shum','shim'], #some mask...
+            'five': numpy.array([1,2,3,4], dtype=numpy.float128)})
+        nf = tempfile.NamedTemporaryFile()
+        DF2Excel().write(a, nf)
+        nf.seek(0,0)
+        nf.flush()
+        def inner():
+            b = DF2Excel().read(nf.name, columns_to_include=['nosuchcolumn'])
+            nf.close()
+        self.assertRaises(ValueError, inner)
+
+    def test_raises_on_non_existant_all(self):
+        a = DataFrame ( {
+            "one": [1, 2,3,4], #mask = False
+            'two': [1.0,2.0,3.0,'SHu'], #mask = True
+            'three': [True, True, False, False], #no mask set... = False
+             'four': ['sha','shu','shum','shim'], #some mask...
+            'five': numpy.array([1,2,3,4], dtype=numpy.float128)})
+        nf = tempfile.NamedTemporaryFile()
+        DF2Excel().write(a, nf)
+        nf.seek(0,0)
+        nf.flush()
+        def inner():
+            b = DF2Excel().read(nf.name, columns_to_include=['nosuchcolumn','four'])
+            nf.close()
+        self.assertRaises(ValueError, inner)
 
 def dataFrameSpeedTest():
     fn = "/k/imt/experiments/20090610_WPMY1-GW501516_BowTieAlignment/out/081208_s_7_seq_GIC-1.txt.stdout"
